/* 
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.samples.graphics;

import com.android.samples.R;

import android.app.Activity;
import android.content.Context;
import android.graphics.*;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;

public class ColorMatrixSample extends GraphicsActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(new SampleView(this));
    }
    
    private static class SampleView extends View {
        private Paint mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        private ColorMatrix mCM = new ColorMatrix();
        private Bitmap mBitmap;
        private float mSaturation;
        private float mAngle;

        public SampleView(Context context) {
            super(context);

            mBitmap = BitmapFactory.decodeResource(context.getResources(),
                                                   R.drawable.balloons);
        }
        
        private static void setTranslate(ColorMatrix cm, float dr, float dg,
                                         float db, float da) {
            cm.set(new float[] {
                   1, 0, 0, 0, dr,
                   0, 1, 0, 0, dg,
                   0, 0, 1, 0, db,
                   0, 0, 0, 1, da });
        }

        @Override protected void onDraw(Canvas canvas) {
            Paint paint = mPaint;
            float x = 20;
            float y = 20;

            canvas.drawColor(Color.WHITE);

            paint.setColorFilter(null);
            canvas.drawBitmap(mBitmap, x, y, paint);

            mCM.setSaturation(mSaturation);
            paint.setColorFilter(new ColorMatrixColorFilter(mCM));
            canvas.drawBitmap(mBitmap, x + mBitmap.getWidth() + 10, y, paint);
            mSaturation += 1.f/10;
            if (mSaturation > 1) {
                mSaturation = -1;
            }
            
            mCM.setRGB2YUV();
            ColorMatrix cm = new ColorMatrix();
            cm.setRotate(0, mAngle);
            mCM.postConcat(cm);
            cm.setYUV2RGB();
            mCM.postConcat(cm);
            paint.setColorFilter(new ColorMatrixColorFilter(mCM));
            canvas.drawBitmap(mBitmap, x, y + mBitmap.getHeight() + 10, paint);
            mAngle += 10;
            if (mAngle > 180) {
                mAngle = -180;
            }
            
            // convert our animated angle [-180...180] to [-255..255]
            float deltaRed = mAngle * 255 / 180;
            setTranslate(cm, deltaRed, 0, 0, 0);
            paint.setColorFilter(new ColorMatrixColorFilter(cm));
            canvas.drawBitmap(mBitmap, x, y + 2*(mBitmap.getHeight() + 10),
                              paint);
            
            invalidate();
        }
    }
}

